
# ========================
# CREDENTIAL STATUS VALUES
# ========================

CRED_VALID   =  1
CRED_INVALID =  0
CRED_FAILED  = -1

# ======================
# USERNAME STATUS VALUES
# ======================
# It's just easier to read when returning multiple values

USERNAME_VALID   = True
USERNAME_INVALID = False
